;    GRDB - debugger for dos with 32-bit extensions
;    Copyright (C) 1997-2004  David Lindauer
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    (for GNU General public license see file COPYING)
;
;    you may contact the author at:  mailto::camille@bluegrass.net
; 
;    or by snail mail at:
;
;    David Lindauer
;    850 Washburn Ave.  Apt #99
;    Louisville, KY 40222
;
;
; OPTIONS.ASM.ASM
;
; Function: Option input and display
;
	;MASM MODE
	.MODEL SMALL
	.386p

ifdef NOFS
NUMOPTS = 13
else
NUMOPTS = 14
endif
include  iversion.inc
include  eprints.inc 
include  einput.inc 
include  emtrap.inc 
include  ebreaks.inc 
include  eints.inc
include  elogging.inc
include  eregs.inc
include  erm32.inc
include  esymbol.inc

; define the symbol RAWA20 to run flat real mode with XMS
;
	PUBLIC doopt,optdword, optdwordcommand,optpure,opthist, optflat0, optsignedimm
        PUBLIC Disassemble32Bit, optdiv0, winshellchk, optflatinit
        PUBLIC optdosio, optass32, optsymname
        PUBLIC ReadOptions, WriteOptions, optstoponkey, writeopts
        PUBLIC optcisym
ifndef nOFS
        public optfs
endif
        extrn wininit : PROC, winrundown : PROC, redump : PROC, rediss : PROC
        extrn w_cmdset: PROC

	.data
;
; optlist, optvals, and optvect MUST be kept in sync
;
optlst  db      "wrfr32zrbknvfif0sohimdddsk"
ifndef NOFS
        db      "fs"
endif
;
writeopts db	0	;true if options have changed
;
optmark	dw	verid	; MUST preced optvals
optvals	label	BYTE
optdword db	1	;flag if user selected 32-bit registers (WR)
optdwordcommand db 0	;flag if user wants flat real mode (FR)
Disassemble32Bit db	1	;32-bit instructions enabled
	TRUE	equ	1
	FALSE	equ	0
optdiv0 db      1       ; these two options DEPEND on VECTLIST
optbrk  db	1	; having the upper bits of the ints set right
			; on a no-param file start
optpure db	1	;native video
optflatinit db	0	;causes init for flat real mode
optflat0 db	0	; flat real mode - default segs to zero
optsignedimm db	0	; set if want to see signed immediates on u command
opthist db	1	;flag if user enabled history
optdosio db     1       ; flag if using dos for command I/O
optass32 db     0       ; flag if disassembly/assembly should be 32 bit native
optstoponkey db 0       ; flag if we should stop various display functions
                        ; when a key is pressed
ifndef NOFS
optfs   db      0       ; flag if full screen mode
endif
optcisym     db 1       ; case insensitive symbol lookups

optsymname      db 0    ; symbol name size, K bytes
;
;
optbottom label byte
;
;
optvect dw      vsetopt,vsetopt,vsetopt
	dw	voptdiv0,voptbrk1b,vsetopt,vsetopt,vsetopt,vsetopt,vsetopt
        dw      vsetopt,vsetopt,vsetopt
ifndef NOFS
        dw      voptfs
endif
optname	db	"grdb.opt",0
	.CODE
;
; for display of states
;
optmsgs	label	BYTE
        db      13,10,"WR - wide registers              ",0
        db      13,10,"FR - flat real commands          ",0
        db      13,10,"32 - enable 386+ instructions    ",0
        db      13,10,"ZR - divide by zero trap         ",0
        db      13,10,"BK - ctrl-break trap             ",0
        db      13,10,"NV - native video                ",0
        db      13,10,"FI - flat real autoinit          ",0
        db      13,10,"F0 - flat real 0 default         ",0
        db      13,10,"SO - signed immediates           ",0
        db      13,10,"HI - command history             ",0
        db      13,10,"MD - msdos I/O                   ",0
        db      13,10,"DD - 32-bit default addressing   ",0
        db      13,10,"SK - stop display on keypress    ",0
ifndef NOFS
        db      13,10,"FS - Full Screen mode            ",0
endif
        db      13,10,"SI - case insensitive symbols    ",0
;
; check for windows shell
;
winshellchk PROC
	push si
	mov	ax,1600h	; check for windows func
	int	2fh		; multiplex int
	pop	si
	cmp	al,00h		; windows not installed if = 0
	je	nowin
	cmp	al,80h		; xms (but no windows) if = 80 h
	je	nowin
	pop	ax		; anything else is windowsxx, drop ret addr
	PRINT_MESSAGE	<10,13,"Feature inactive in windows shells">
	clc
nowin:
	ret
	
winshellchk	ENDP

;
; option command
;
doopt	PROC	
	Call	WadeSpace	; Wade till address
	jnz	optlp
ifndef NOFS
        test    [optfs],255
        jz      nofsx
        mov     al,1
        call    w_cmdset
nofsx:
endif
	mov	si,offset _text : optmsgs	; no args, print all options
	mov	di,offset optvals
	mov	cx,NUMOPTS
polp:
	call	PrintOption
	loop	polp
	call	LoggingStat
	call	initflatreal
optxit:
	clc
	ret
;
; subroutine to print an option and its enabled/disabled value
;
PrintOption:
        mov     bx,si
	call	olMessage
	test	byte ptr [di],-1
	jnz	dotype
	PRINT_MESSAGE	<9,"disabled">
	jmp	dojoin
dotype:
	PRINT_MESSAGE	<9,"enabled">
dojoin:
	inc	di
frs_lp:
	lods	byte ptr cs:[si]
	or	al,al
	jnz	frs_lp
	ret
;
; comes here to set/reset an option
;
optlp:
	cmp	al,13
	je	doopt		; go back and show vals after the set
	cmp	al,'-'		; check for off command
	pushf
	jz	incpos
	cmp	al,'+'
	jne	noinc
incpos:
	inc	si
noinc:
	call	WadeSpace	; now get the option letters
	jz	opterr
	lodsw
	cmp	ah,13
	je	opterr
	mov	cx,NUMOPTS	; search for them
	mov	di,offset optlst
cmplp:
	scasw
	jz	dovect
	loop	cmplp
opterr:
	add	sp,2		; bad opt, exit with no display
        stc
	ret
dovect:
	mov	[writeopts],1	; changing, need disk update
	neg	cx
	add	cx,NUMOPTS
	movzx	ebx,cx
	popf
	lahf
	and	ah,40h
	xor	ah,40h
	shr	ah,6        	; ah = opt new val
	call	[optvect + ebx*2] ; call handler
	call	WadeSpace
	jmp	optlp    	; get another
doopt	ENDP
;
; option routines
; 
; this one is for basic bools
;
vsetopt PROC
	mov	[bx + optvals],ah
	ret
vsetopt	ENDP
ifndef NOFS
voptfs  PROC
        cmp     ah,[bx + optvals]
        jz      voptfsx
        test    ah,ah
        jz      fsout
        mov     [bx + optvals],1
        push    si
        push    di
        call    wininit
        call    DisplayRegisters
        call    redump
        call    rediss
        pop     di
        pop     si
        ret
fsout:
        push    bx
        push    si
        push    di
        call    winrundown
        pop     di
        pop     si
        pop     bx
        mov     [bx + optvals],0
voptfsx:
        ret
voptfs  ENDP
endif
;
; following this we have interrupt enables/disables.
; they run through VECLIST and set the high bit to match the
; bool value
;         
voptbrk1b	PROC
	mov	al,1bh
	jmp	vopttraps
voptbrk1b	ENDP
voptdiv0	PROC
	mov	al,0
voptdiv0	ENDP
vopttraps	PROC
	mov	[bx + optvals],ah
	mov	bl,ah
vopttraps	ENDP
voptrefresh	PROC
	push	si
	mov	si,offset veclist
	call	SetVectAttrib
	pop	si
	ret
voptrefresh	ENDP

;
; write options to disk file
;
WriteOptions	PROC
	mov	[optmark],verid
	test	[writeopts],0ffh
	jz	wo_x
	call	optcreat
	jc	wo_x
	mov	dx,offset optmark
        mov     cx,offset optbottom - offset optmark
	mov	ah,40h
	int	21h
	jc	wo_x2
	mov	[writeopts],0
wo_x2:
	call	optclose
wo_x:
	ret
WriteOptions	ENDP
;
; read options from disk file
;
ReadOptions	PROC
	mov	[writeopts],1
	mov	[optmark],-1
	call	optopen
	jc	ro_x
	mov	dx,offset optmark
	mov	cx,2
	mov	ah,3fh
	int	21h
	jc	ro_x2
	cmp	[optmark],verid
	jne	ro_x2
	mov	dx,offset optmark+2
        mov     cx,offset optbottom - offset optmark - 2 
	mov	ah,3fh
	int	21h
	jc	ro_x2
        cmp     ax,offset optbottom - offset optmark - 2
	jne	ro_x2
	push	bx
	mov	[writeopts],0   ; the option file is read
	mov	bl,[optdiv0]	; now set the attrib bits in veclist
	mov	al,0
	call	voptRefresh
	mov	bl,[optbrk]
	mov	al,1bh
	call	voptRefresh
	call	initflatreal
	pop	bx
nd1bfix:
ro_x2:
	call	optclose
ro_x:
	ret
ReadOptions	ENDP


;
; generi disk file stuff.  Should probably merge with loader and logger
; routines...
;
OptOpen	PROC
	mov	ax,3d02h
	mov	dx,offset optname
	int	21h
	mov	bx,ax
	ret
OptOpen	ENDP
OptCreat	PROC
	mov	ax,3c00h
	mov	cx,0
	mov	dx,offset optname
	int	21h
	mov	bx,ax
	ret
OptCreat	ENDP
OptClose	PROC
	mov	ah,3eh
	int	21h
	ret
OptClose	ENDP

end
